<?xml version="1.0" encoding="UTF-8"?>
<!-- ILCD Format Version 1.1_SNAPSHOT Tools Build 1016 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:process="http://lca.jrc.it/ILCD/Process"
                xmlns:flow="http://lca.jrc.it/ILCD/Flow" xmlns:common="http://lca.jrc.it/ILCD/Common"
                xmlns:epd="http://www.iai.kit.edu/EPD/2013" xmlns:epd2="http://www.indata.network/EPD/2019"
                xmlns:matml="http://www.matml.org/" xmlns:exslt="http://exslt.org/common">

    <xsl:import href="validate-common-1.2.xsl"/>
    <xsl:import href="validate-references-1.2.xsl"/>

    <xsl:output indent="no" method="text"/>

    <xsl:variable name="version" select="'1.1'"/>

    <xsl:variable name="allowedModules" select="'|A1|A2|A3|A4|A5|A1-A3|B1|B2|B3|B4|B5|B6|B7|C1|C2|C3|C4|D|'"/>

    <xsl:variable name="fancyMessages" select="false()"/>

    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>


    <xsl:template match="process:processDataSet">
        <xsl:call-template name="checkEPD12Version"/>
        <xsl:call-template name="checkFieldsPresence"/>
        <xsl:call-template name="checkModules"/>
        <xsl:call-template name="checkScenarios"/>
        <xsl:call-template name="checkBackgroundData"/>
    </xsl:template>

    <xsl:template name="checkEPD12Version">
        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="/process:processDataSet[@epd2:epd-version='1.2']"/>
            <xsl:with-param name="elementDesc">On root element, attribute @epd-version with value "1.2"</xsl:with-param>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="flow:flowDataSet">
    	<xsl:variable name="uuid" select="/flow:flowDataSet/flow:flowInformation/flow:dataSetInformation/common:UUID/text()"/>
    	<!--  exclude the LCI indicator flows -->
    	<xsl:if test="$uuid!='20f32be5-0398-4288-9b6d-accddd195317' and
    				  $uuid!='fb3ec0de-548d-4508-aea5-00b73bf6f702' and
    				  $uuid!='53f97275-fa8a-4cdd-9024-65936002acd0' and
    				  $uuid!='ac857178-2b45-46ec-892a-a9a4332f0372' and
    				  $uuid!='1421caa0-679d-4bf4-b282-0eb850ccae27' and
    				  $uuid!='06159210-646b-4c8d-8583-da9b3b95a6c1' and
    				  $uuid!='c6a1f35f-2d09-4f54-8dfb-97e502e1ce92' and
    				  $uuid!='64333088-a55f-4aa2-9a31-c10b07816787' and
    				  $uuid!='89def144-d39a-4287-b86f-efde453ddcb2' and
    				  $uuid!='3cf952c8-f3a4-461d-8c96-96456ca62246' and
    				  $uuid!='430f9e0f-59b2-46a0-8e0d-55e0e84948fc' and
    				  $uuid!='b29ef66b-e286-4afa-949f-62f1a7b4d7fa' and
    				  $uuid!='3449546e-52ad-4b39-b809-9fb77cea8ff6' and
    				  $uuid!='a2b32f97-3fc7-4af2-b209-525bc6426f33' and
    				  $uuid!='d7fe48a5-4103-49c8-9aae-b0b5dfdbd6ae' and
    				  $uuid!='59a9181c-3aaf-46ee-8b13-2b3723b6e447' and
    				  $uuid!='4da0c987-2b76-40d6-9e9e-82a017aaaf29' and
    				  $uuid!='98daf38a-7a79-46d3-9a37-2b7bd0955810'">
            <xsl:call-template name="checkFlowReferenceFlowPropertyPresence"/>
            <xsl:call-template name="checkFlowInvalidFlowProperties"/>
    	</xsl:if>
        <xsl:call-template name="checkMaterialProperties"/>
    </xsl:template>


    <xsl:template name="checkMaterialProperties">
        <xsl:variable name="matMLContent" select="/flow:flowDataSet/flow:flowInformation/flow:dataSetInformation/common:other/matml:MatML_Doc"/>

        <!-- check that a value and the correct unit are declared for material properties -->
        <xsl:for-each select="$matMLContent/matml:Metadata/matml:PropertyDetails">
            <xsl:variable name="index" select="@id"/>
            <xsl:variable name="originalName" select="matml:Name/text()"/>
            <xsl:variable name="name" select="translate($originalName,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>

            <!-- for each declared property, we'll look for the expected unit if that can't be found, we'll complain -->
            <!-- there are two ways to express the unit - either as simple text ($expectedUnitSimple) or in MatML notation -->
            <xsl:variable name="expectedUnitSimple">
                <xsl:choose>
                    <xsl:when test="$name='bulk density'">kg/m^3</xsl:when>
                    <xsl:when test="$name='grammage'">kg/m^2</xsl:when>
                    <xsl:when test="$name='gross density'">kg/m^3</xsl:when>
                    <xsl:when test="$name='layer thickness'">m</xsl:when>
                    <xsl:when test="$name='productiveness'">m^2</xsl:when>
                    <xsl:when test="$name='linear density'">kg/m</xsl:when>
                    <xsl:when test="$name='conversion factor to 1 kg'">-</xsl:when>
                    <xsl:otherwise>#unsupported#</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>

<!--            <xsl:message>MP: <xsl:value-of select="$name"/></xsl:message>-->

            <xsl:variable name="expectedUnitMatMlEval">
                <xsl:choose>
                    <xsl:when test="$name='bulk density'">
                        <!-- kg/m^3 -->
                        <!--
                        <mat:Units name="kg/m^3" description="kilograms per cubic metre">
                           <mat:Unit>
                              <mat:Name>kg</mat:Name>
                           </mat:Unit>
                           <mat:Unit power="-3">
                              <mat:Name>m</mat:Name>
                           </mat:Unit>
                        </mat:Units>
                        -->
                        <!-- kg/m^3 -->
                        <xsl:if test="matml:Units/@name=$expectedUnitSimple and
                                      count(matml:Units/matml:Unit)=2 and
                                      matml:Units/matml:Unit[matml:Name/text()='kg' and (not(@power) or @power='1')] and
                                      matml:Units/matml:Unit[matml:Name/text()='m' and @power='-3']">
                            <xsl:value-of select="string(true())"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:when test="$name='grammage'">
                        <!-- kg/m^2 -->
                        <!-- kg/m^3 -->
                        <xsl:if test="matml:Units/@name=$expectedUnitSimple and
                                      count(matml:Units/matml:Unit)=2 and
                                      matml:Units/matml:Unit[matml:Name/text()='kg' and (not(@power) or @power='1')] and
                                      matml:Units/matml:Unit[matml:Name/text()='m' and @power='-2']">
                            <xsl:value-of select="string(true())"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:when test="$name='gross density'">
                        <!-- kg/m^3 -->
                        <xsl:if test="matml:Units/@name=$expectedUnitSimple and
                                      count(matml:Units/matml:Unit)=2 and
                                      matml:Units/matml:Unit[matml:Name/text()='kg' and (not(@power) or @power='1')] and
                                      matml:Units/matml:Unit[matml:Name/text()='m' and @power='-3']">
                            <xsl:value-of select="string(true())"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:when test="$name='layer thickness'">
                        <!-- m -->
                        <xsl:if test="matml:Units/@name=$expectedUnitSimple and
                                      count(matml:Units/matml:Unit)=1 and
                                      matml:Units/matml:Unit[matml:Name/text()='m' and (not(@power) or @power='1')]">
                            <xsl:value-of select="string(true())"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:when test="$name='productiveness'">
                        <!-- m^2 -->
                        <xsl:if test="matml:Units/@name=$expectedUnitSimple and
                                      count(matml:Units/matml:Unit)=1 and
                                      matml:Units/matml:Unit[matml:Name/text()='m' and @power='2']">
                            <xsl:value-of select="string(true())"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:when test="$name='linear density'">
                        <!-- kg/m -->
                        <xsl:if test="matml:Units/@name=$expectedUnitSimple and
                                      count(matml:Units/matml:Unit)=2 and
                                      matml:Units/matml:Unit[matml:Name/text()='kg' and (not(@power) or @power='1')] and
                                      matml:Units/matml:Unit[matml:Name/text()='m' and @power='-1']">
                            <xsl:value-of select="string(true())"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:when test="$name='conversion factor to 1 kg'">
                        <xsl:if test="matml:Units/@name=$expectedUnitSimple and
                                      count(matml:Units/matml:Unit)=1 and
                                      matml:Units/matml:Unit[matml:Name/text()='-' and (not(@power) or @power='1')]">
                            <xsl:value-of select="string(true())"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:otherwise>#unsupported#</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>

<!--            <xsl:message>MATML eval: <xsl:value-of select="$expectedUnitMatMlEval"/></xsl:message>-->

            <xsl:choose>
                <xsl:when test="$expectedUnitSimple='#unsupported#'">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="type" select="'warning'"/>
                        <xsl:with-param name="message">Unsupported material property '<xsl:value-of select="$name"/>' found.</xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:if test="matml:Units/@name!=$expectedUnitSimple or (matml:Units/matml:Unit/matml:Name/text()!=$expectedUnitSimple and $expectedUnitMatMlEval!='true')">
                        <xsl:call-template name="validationEvent">
                            <xsl:with-param name="type" select="'error'"/>
                            <xsl:with-param name="message">The expected unit '<xsl:value-of select="$expectedUnitSimple"/>' for material property '<xsl:value-of select="$name"/>' could not be found.</xsl:with-param>
                        </xsl:call-template>
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>

            <!-- make sure a value is declared -->
            <xsl:if test="count($matMLContent/matml:Material/matml:BulkDetails/matml:PropertyData[@property=$index])=0">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="type" select="'error'"/>
                    <xsl:with-param name="message">No value has been declared for material property '<xsl:value-of select="$name"/>'.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
            <xsl:if test="string(number($matMLContent/matml:Material/matml:BulkDetails/matml:PropertyData[@property=$index]/matml:Data/text()))='NaN'">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="type" select="'error'"/>
                    <xsl:with-param name="message">No valid number value has been declared for material property '<xsl:value-of select="$name"/>'.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>

            <!-- make sure a discrete value is declared -->
            <xsl:if test="$matMLContent/matml:Material/matml:BulkDetails/matml:PropertyData[@property=$index]/matml:Data/@format!='integer' and $matMLContent/matml:Material/matml:BulkDetails/matml:PropertyData[@property=$index]/matml:Data/@format!='float'">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="type" select="'error'"/>
                    <xsl:with-param name="message">A discrete integer or float value must be declared for material property '<xsl:value-of select="$name"/>', but format <xsl:value-of select="$matMLContent/matml:Material/matml:BulkDetails/matml:PropertyData[@property=$index]/matml:Data/@format"/> is specified.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>

        </xsl:for-each>

    </xsl:template>

    <!-- a reference flow property must be present and from a list of allowed ones -->
    <xsl:template name="checkFlowReferenceFlowPropertyPresence">
        
        <xsl:variable name="numberOfDeclaredFlowProperties" select="count(/flow:flowDataSet/flow:flowProperties/flow:flowProperty)"/>
        
        <xsl:choose>
            <xsl:when test="$numberOfDeclaredFlowProperties=0">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="message">No flow properties are declared for the reference flow.</xsl:with-param>
                </xsl:call-template>                              
            </xsl:when>
            <xsl:otherwise>
                <xsl:if test="count(/flow:flowDataSet/flow:flowProperties/flow:flowProperty[@dataSetInternalID=/flow:flowDataSet/flow:flowInformation/flow:quantitativeReference/flow:referenceToReferenceFlowProperty])=0">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">No reference flow property is declared for the reference flow.</xsl:with-param>
                    </xsl:call-template>                                                  
                </xsl:if>

                <!-- reference FP UUID must be from a list of allowed ones -->
                <xsl:variable name="referenceFPUUID" select="/flow:flowDataSet/flow:flowProperties/flow:flowProperty[@dataSetInternalID=/flow:flowDataSet/flow:flowInformation/flow:quantitativeReference/flow:referenceToReferenceFlowProperty]/flow:referenceToFlowPropertyDataSet/@refObjectId"/>

                <xsl:if test="$referenceFPUUID != '93a60a56-a3c8-11da-a746-0800200b9a66' and
                            $referenceFPUUID != '93a60a56-a3c8-22da-a746-0800200c9a66' and
                            $referenceFPUUID != '93a60a56-a3c8-19da-a746-0800200c9a66' and
                            $referenceFPUUID != '838aaa23-0117-11db-92e3-0800200c9a66' and
                            $referenceFPUUID != '01846770-4cfe-4a25-8ad9-919d8d378345' and
                            $referenceFPUUID != '93a60a56-a3c8-21da-a746-0800200c9a66' and
                            $referenceFPUUID != 'c0447923-0e60-4b3c-97c2-a86dddd9eea5' and
                            $referenceFPUUID != '838aaa20-0117-11db-92e3-0800200c9a66' and
                            $referenceFPUUID != '93a60a56-a3c8-14da-a746-0800200c9a66' and
                            $referenceFPUUID != '93a60a56-a3c8-11da-a746-0800200c9a66'">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">An invalid reference flow property (<xsl:value-of
                                select="$referenceFPUUID"/>) is declared for the reference flow. The reference flow property must be one of Mass, Volume, Area, Length Number of pieces, Area*time, Duration, Goods transport (mass*distance), Gross calorific value, Net calorific value.</xsl:with-param>
                    </xsl:call-template>
                </xsl:if>


            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <!-- if any declared flow properties are actually unit groups, we'll raise an error -->
    <xsl:template name="checkFlowInvalidFlowProperties">
        <xsl:for-each select="/flow:flowDataSet/flow:flowProperties/flow:flowProperty/flow:referenceToFlowPropertyDataSet">
            <xsl:variable name="referencedUUID" select="translate(@refObjectId, 'ABCDEF', 'abcdef')"/>

            <xsl:if test="$referencedUUID = 'f170abd3-f010-45f4-8e7c-9871a5c0421b' or
                        $referencedUUID = '93a60a57-a4c8-11da-a746-0800200c9a66' or
                        $referencedUUID = '93a60a57-a3c8-23da-a746-0800200c9a66' or
                        $referencedUUID = '93a60a57-a3c8-20da-a746-0800200c9a66' or
                        $referencedUUID = '93a60a57-a3c8-18da-a746-0800200c9a66' or
                        $referencedUUID = '93a60a57-a3c8-16da-a746-0800200c9a66' or
                        $referencedUUID = '93a60a57-a3c8-12da-a746-0800200c9a66' or
                        $referencedUUID = '93a60a57-a3c8-11da-a746-0800200c9a66' or
                        $referencedUUID = '5beb6eed-33a9-47b8-9ede-1dfe8f679159' or
                        $referencedUUID = '59f191d6-5dd3-4553-af88-1a32accfe308' or
                        $referencedUUID = '03cc996d-7177-4738-aa32-1572a5172ccc' or
                        $referencedUUID = 'ad38d542-3fe9-439d-9b95-2f5f7752acaf' or
                        $referencedUUID = 'cd950537-0a98-4044-9ba7-9f9a68d0a504' or
                        $referencedUUID = 'de5104d8-3de0-4218-a29d-b7123ce9ca3c' or
                        $referencedUUID = 'c20a03d7-bd90-4569-bc94-66cfd364dfc8' or
                        $referencedUUID = '838aaa22-0117-11db-92e3-0800200c9a66' or
                        $referencedUUID = 'b3a220db-0613-42ab-84bc-b153d339cb0d' or
                        $referencedUUID = '838aaa21-0117-11db-92e3-0800200c9a66' or
                        $referencedUUID = '59f191d6-5dd3-4553-af88-1a32accfe308' or
                        $referencedUUID = '5454b231-270e-45e6-89b2-7f4f3e482245' or
                        $referencedUUID = '1acf7c0f-aa9d-4022-a4b7-fe0b54ea0b91' or
                        $referencedUUID = '934110e3-baf4-49e9-992c-3d8109a6aafb' or
                        $referencedUUID = '3620148f-c5db-48ce-9065-a10092089aca' or
                        $referencedUUID = 'ff8ed45d-bbfb-4531-8c7b-9b95e52bd41d' or
                        $referencedUUID = '93a60a57-a3c8-23da-a746-0800200c9a66'">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="message">An invalid flow property (<xsl:value-of
                            select="$referencedUUID"/>) is declared for the reference flow. The specified UUID actually belongs to a unit group dataset which may not be declared here! This is strictly a violation of the ILCD format specification.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>


    <xsl:template name="checkModules">
        <xsl:for-each select="//common:other/epd:amount">
            <xsl:variable name="module" select="concat('|', @epd:module, '|')"/>
            <!-- check module name -->
            <xsl:choose>
                <xsl:when test="$module='||'">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">No module declared for value <xsl:value-of select="./text()"/></xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:if test="not(contains($allowedModules, $module))">
                        <xsl:call-template name="validationEvent">
                            <xsl:with-param name="message">Module name '<xsl:value-of select="@epd:module"/>' is invalid. Only one of <xsl:value-of select="$allowedModules"/> is allowed.</xsl:with-param>                            
                        </xsl:call-template>
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>            
        </xsl:for-each>
        
        <!-- if D is declared, also C must be declared -->
        <xsl:if test="count(//common:other//epd:amount[@epd:module='D' and count(preceding-sibling::epd:amount[starts-with(@epd:module,'C')])=0 and count(following-sibling::epd:amount[starts-with(@epd:module,'C')])=0])!=0">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="message">A value for module D has been declared for at least one indicator without declaring a value for module C, which is required in this case.</xsl:with-param>                            
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="checkBackgroundData">
        <xsl:if test="count(/process:processDataSet/process:modellingAndValidation/process:dataSourcesTreatmentAndRepresentativeness/process:referenceToDataSource[@refObjectId='28d74cc0-db8b-4d7e-bc44-5f6d56ce0c4a' or @refObjectId='b497a91f-e14b-4b69-8f28-f50eb1576766'])=0">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="message">No mandatory background database (GaBi, ecoinvent) has been declared under datasources.</xsl:with-param>                            
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template name="checkScenarios">
        <!-- scenarios -->
        <!-- 1. scenario declarations in metadata must match scenarios used in results - no more and no less -->
        <!-- 2. exactly one default scenario per group -->
        <!-- 3. scenarios used for one module must be consistently declared on each indicator -->

<!--        <xsl:message>DEBUG checking scenarios</xsl:message>-->

        <!--    - collect declared scenarios in metadata
                - collect declared scenarios in results -->
        <xsl:variable name="declaredScenarios" select="process:processInformation/process:dataSetInformation/common:other/epd:scenarios"/>
        <xsl:variable name="usedScenarios" select="process:exchanges/process:exchange/common:other/epd:amount[@epd:scenario]"/>

        <!--    1. scenarios declared in metadata section must be referenced in results  -->
        <xsl:for-each select="exslt:node-set($declaredScenarios)/epd:scenario">
            <xsl:variable name="declaredName" select="@epd:name"/>

<!--            <xsl:message>DEBUG declared scenario: <xsl:value-of select="@epd:group"/> # <xsl:value-of select="@epd:name"/></xsl:message>-->
<!--            <xsl:message>DEBUG <xsl:value-of select="count(exslt:node-set($usedScenarios)[@epd:scenario=$declaredName])"/></xsl:message>-->

            <xsl:if test="count(exslt:node-set($usedScenarios)[@epd:scenario=$declaredName])=0">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="message">Scenario '<xsl:value-of select="$declaredName"/>' has been declared in metadata section but is never referenced.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
        </xsl:for-each>

        <!--    1. scenarios referenced results must have been declared in metadata section  -->
        <xsl:for-each select="exslt:node-set($usedScenarios)">
            <xsl:variable name="referencedName" select="@epd:scenario"/>

            <xsl:if test="count(exslt:node-set($declaredScenarios)/epd:scenario[@epd:name=$referencedName])=0">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="message">Scenario '<xsl:value-of select="$referencedName"/>' is referenced in results but has not been declared in metadata section.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
        </xsl:for-each>

        <!--    2. exactly one default scenario per group  -->
        <xsl:for-each select="exslt:node-set($declaredScenarios)/epd:scenario/@epd:group">
<!--            <xsl:message>DEBUG checking scenario group <xsl:value-of select="."/></xsl:message>-->
            <xsl:variable name="declaredGroup" select="."/>
            <xsl:variable name="defaultCount" select="count(exslt:node-set($declaredScenarios)/epd:scenario[@epd:group=$declaredGroup and @epd:default=true()])"/>
            <xsl:choose>
                <xsl:when test="$defaultCount = 0">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">No default scenario declared for scenario group '<xsl:value-of select="$declaredGroup"/>'. Exactly one default scenario must be declared per group.</xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when test="$defaultCount &gt; 1">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">More than one default scenario declared for scenario group '<xsl:value-of select="$declaredGroup"/>'. Exactly one default scenario must be declared per group.</xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>
        <!--    2. now check for those w/o group attribute  -->
        <xsl:for-each select="exslt:node-set($declaredScenarios)/epd:scenario[not(@epd:group)]">
            <xsl:variable name="scenarioName" select="@epd:name"/>
<!--            <xsl:message>DEBUG checking default scenario group for <xsl:value-of select="$scenarioName"/></xsl:message>-->
            <xsl:variable name="defaultCount" select="count(exslt:node-set($declaredScenarios)/epd:scenario[not(@epd:group) and @epd:default=true()])"/>
            <xsl:choose>
                <xsl:when test="$defaultCount = 0">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">No default scenario declared, scenario name is '<xsl:value-of select="$scenarioName"/>'. Exactly one default scenario must be declared.</xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when test="$defaultCount &gt; 1">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">More than one default scenario declared, scenario name is '<xsl:value-of select="$scenarioName"/>'. Exactly one default scenario must be declared.</xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>

        <!--  TODO  3. scenarios used for one module must be consistently declared on each indicator -->

    </xsl:template>

    <xsl:template name="checkFieldsPresence">
        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:dataSetInformation/common:UUID"/>
            <xsl:with-param name="elementDesc" select="'UUID'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:dataSetInformation/process:name/process:baseName"/>
            <xsl:with-param name="elementDesc" select="'name'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

       <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:quantitativeReference/process:referenceToReferenceFlow"/>
            <xsl:with-param name="elementDesc" select="'reference to reference flow in section quantitative reference'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:time/common:referenceYear"/>
            <xsl:with-param name="elementDesc" select="'reference year'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:time/common:dataSetValidUntil"/>
            <xsl:with-param name="elementDesc" select="'data set valid until'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:geography/process:locationOfOperationSupplyOrProduction/@location"/>
            <xsl:with-param name="elementDesc" select="'location'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkValue">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:LCIMethodAndAllocation/process:typeOfDataSet"/>
            <xsl:with-param name="value1" select="'EPD'"/>
            <xsl:with-param name="elementDesc" select="'type of dataset'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:LCIMethodAndAllocation/common:other/epd:subType"/>
            <xsl:with-param name="elementDesc" select="'subtype of dataset'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkValue">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:LCIMethodAndAllocation/common:other/epd:subType"/>
            <xsl:with-param name="value1" select="'generic dataset'"/>
            <xsl:with-param name="value2" select="'representative dataset'"/>
            <xsl:with-param name="value3" select="'average dataset'"/>
            <xsl:with-param name="value4" select="'specific dataset'"/>
            <xsl:with-param name="value5" select="'template dataset'"/>
            <xsl:with-param name="elementDesc" select="'type of dataset'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:validation/process:review/common:referenceToNameOfReviewerAndInstitution"/>
            <xsl:with-param name="elementDesc" select="'Reviewer name and institution'"/>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element"
                select="process:administrativeInformation/process:dataEntryBy/common:referenceToDataSetFormat[@refObjectId = 'a97a0155-0234-4b87-b4ce-a45da52f2a40']"/>
            <xsl:with-param name="elementDesc" select="'Declaration of ILCD format 1.1'"/>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element"
                select="process:administrativeInformation/process:dataEntryBy/common:referenceToDataSetFormat[@refObjectId = 'a29449fd-aa2f-4de8-b5d7-4b06b43c6fde']"/>
            <xsl:with-param name="elementDesc" select="'Declaration of EPD Data Format Extensions 1.2'"/>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:administrativeInformation/process:publicationAndOwnership/common:dataSetVersion"/>
            <xsl:with-param name="elementDesc" select="'dataset version'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:administrativeInformation/process:publicationAndOwnership/common:referenceToOwnershipOfDataSet"/>
            <xsl:with-param name="elementDesc" select="'owner of dataset'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:if test="process:modellingAndValidation/process:LCIMethodAndAllocation/common:other/epd:subType!='generic dataset' and process:modellingAndValidation/process:LCIMethodAndAllocation/common:other/epd:subType!='representative dataset'">
            <!-- referenceToOriginalEPD -->
            <xsl:call-template name="checkPresence">
                <xsl:with-param name="element" select="process:modellingAndValidation/process:dataSourcesTreatmentAndRepresentativeness/common:other/epd2:referenceToOriginalEPD[@refObjectId]"/>
                <xsl:with-param name="elementDesc" select="'reference to original EPD'"/>
                <xsl:with-param name="messagePrefix"/>
            </xsl:call-template>

            <!-- publicationDate -->
            <xsl:call-template name="checkPresence">
                <xsl:with-param name="element" select="process:processInformation/process:time/common:other/epd2:publicationDateOfEPD"/>
                <xsl:with-param name="elementDesc" select="'publication date of EPD'"/>
                <xsl:with-param name="messagePrefix"/>
            </xsl:call-template>

            <!-- registration number -->
            <xsl:call-template name="checkPresence">
                <xsl:with-param name="element" select="process:administrativeInformation/process:publicationAndOwnership/common:registrationNumber"/>
                <xsl:with-param name="elementDesc" select="'registration number'"/>
                <xsl:with-param name="messagePrefix"/>
            </xsl:call-template>

        </xsl:if>

        <!-- issuer/registration authority -->
        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:administrativeInformation/process:publicationAndOwnership/common:referenceToRegistrationAuthority[@refObjectId]"/>
            <xsl:with-param name="elementDesc" select="'issuer of the dataset/registration authority (usually the Program Operator)'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:apply-templates/>

    </xsl:template>

    <xsl:template match="*[local-name()='classificationInformation']" priority="1000"/>

</xsl:stylesheet>
